package com.unis.module.coding.controller.admin.task;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unis.framework.common.pojo.CommonResult;
import com.unis.module.coding.controller.admin.task.vo.*;
import com.unis.module.coding.service.task.TaskService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

import static com.unis.framework.common.pojo.CommonResult.success;

@Tag(name = "管理后台 - app 任务列表")
@RestController
@RequestMapping("/delivery/task")
@Validated
public class TaskController {

    @Resource
    private TaskService taskService;

    @PostMapping("/getTaskList")
    @Operation(summary = "获取我的任务列表")
    public CommonResult<IPage<TaskRespVO>> getTaskList(@Valid @RequestBody TaskPageReqVO pageReqVO) {
        return success(taskService.getTaskList(pageReqVO));
    }
    @PostMapping("/getTasks")
    @Operation(summary = "领取任务(多个):参数:kqIds列表,hyState:核验状态(0:待查验,1:查验通过,2:查验不通过3:核验中)")
    public CommonResult<Boolean> getTasks(@RequestBody TaskPageReqVO reqVO) {
        return success(taskService.getTasks(reqVO));
    }
    @PostMapping("/hyKqxx")
    @Operation(summary = "核验通过/不通过:参数:kqIds列表,hyState:核验状态(0:待查验,1:查验通过,2:查验不通过3:核验中)")
    public CommonResult<Boolean> hyKqxx(@RequestBody TaskPageReqVO reqVO) {
        return success(taskService.hyKqxx(reqVO));
    }

    @PostMapping("/getCaxxByKqId")
    @Operation(summary = "获取仓房信息(根据库区id 查询仓房列表)")
    public CommonResult<List<TaskCaxxRespVO>> getCaxxByKqId(@RequestBody TaskPageReqVO reqVO) {
        return success(taskService.getCaxxByKqId(reqVO));
    }

    @PostMapping("/getKqxxByKqId")
    @Operation(summary = "获取库区信息(根据库区id)")
    public CommonResult<TaskKqxxRespVO> getKqxxByKqId(@RequestBody TaskPageReqVO reqVO) {
        return success(taskService.getKqxxByKqId(reqVO));
    }
    @PostMapping("/getCwxxByKqId")
    @Operation(summary = "获取财务信息(根据库区id)")
    public CommonResult<TaskCwxxRespVO> getCwxxByKqId(@RequestBody TaskPageReqVO reqVO) {
        return success(taskService.getCwxxByKqId(reqVO));
    }
    @PostMapping("/getDkxxByKqId")
    @Operation(summary = "获取打卡信息(根据库区id&userId)")
    public CommonResult<TaskDkxxRespVO> getDkxxByKqId(@RequestBody TaskPageReqVO reqVO) {
        return success(taskService.getDkxxByKqId(reqVO));
    }
    @PostMapping("/setDkxxByKqId")
    @Operation(summary = "打卡(传 id 是变更(传图片),不传 id 是新增")
    public CommonResult<Boolean> setDkxxByKqId(@RequestBody TaskDkxxSaveVO saveVO) {
        return success(taskService.setDkxxByKqId(saveVO));
    }
}