package com.unis.module.coding.service.dwxx;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.yitter.idgen.YitIdHelper;
import com.unis.framework.common.util.business.PermissionUtils;
import com.unis.framework.security.core.LoginUser;
import com.unis.framework.security.core.util.AreaUtils;
import com.unis.framework.security.core.util.SecurityFrameworkUtils;
import com.unis.module.coding.controller.admin.delivery.vo.*;
import com.unis.module.coding.dal.dataobject.bank.BankDO;
import com.unis.module.coding.dal.dataobject.caxx.CaxxDO;
import com.unis.module.coding.dal.dataobject.codingcaxx.CodingCaxxDO;
import com.unis.module.coding.dal.dataobject.codingdwxx.CodingDwxxDO;
import com.unis.module.coding.dal.dataobject.codingkqxx.CodingKqxxDO;
import com.unis.module.coding.dal.dataobject.cwxx.CwxxDO;
import com.unis.module.coding.dal.dataobject.kqxx.KqxxDO;
import com.unis.module.coding.dal.dataobject.province.ProvinceDO;
import com.unis.module.coding.dal.mysql.bank.BankMapper;
import com.unis.module.coding.dal.mysql.caxx.CaxxMapper;
import com.unis.module.coding.dal.mysql.codingcaxx.CodingCaxxMapper;
import com.unis.module.coding.dal.mysql.codingdwxx.CodingDwxxMapper;
import com.unis.module.coding.dal.mysql.codingkqxx.CodingKqxxMapper;
import com.unis.module.coding.dal.mysql.cwxx.CwxxMapper;
import com.unis.module.coding.dal.mysql.cwxxyear.CwxxYearMapper;
import com.unis.module.coding.dal.mysql.kqxx.KqxxMapper;
import com.unis.module.coding.dal.mysql.province.ProvinceMapper;
import com.unis.module.core.enums.XZQHLEVEL;
import com.unis.module.core.enums.YXZT;
import com.unis.module.system.dal.dataobject.arealevel.AreaLevelDO;
import com.unis.module.system.dal.dataobject.permission.UserRoleDO;
import com.unis.module.system.dal.dataobject.user.AdminUserDO;
import com.unis.module.system.dal.mysql.arealevel.AreaLevelMapper;
import com.unis.module.system.dal.mysql.permission.UserRoleMapper;
import com.unis.module.system.dal.mysql.user.AdminUserMapper;
import com.unis.module.system.enums.permission.RoleIdEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import com.unis.module.coding.controller.admin.dwxx.vo.*;
import com.unis.module.coding.dal.dataobject.dwxx.DwxxDO;
import com.unis.framework.common.pojo.PageResult;
import com.unis.framework.common.util.object.BeanUtils;

import com.unis.module.coding.dal.mysql.dwxx.DwxxMapper;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.unis.framework.common.exception.util.ServiceExceptionUtil.exception;
import static com.unis.module.core.enums.ErrorCodeConstants.DWXX_LOGIN_USER_EXISTS;
import static com.unis.module.core.enums.ErrorCodeConstants.DWXX_NOT_EXISTS;
import static com.unis.module.infra.enums.ErrorCodeConstants.ERROR;

/**
 * 单位信息 Service 实现类
 *
 * @author 超级管理员
 */
@Slf4j
@Service
@Validated
public class DwxxServiceImpl implements DwxxService {

    @Resource
    private DwxxMapper dwxxMapper;
    @Resource
    private UserRoleMapper userRoleMapper;
    @Resource
    private AdminUserMapper adminUserMapper;
    @Resource
    private CodingKqxxMapper codingKqxxMapper;
    @Resource
    private CodingCaxxMapper codingCaxxMapper;
    @Resource
    private CodingDwxxMapper codingDwxxMapper;
    @Resource
    private KqxxMapper kqxxMapper;
    @Resource
    private CaxxMapper caxxMapper;
    @Resource
    private AreaLevelMapper areaLevelMapper;
    @Resource
    private ProvinceMapper provinceMapper;
    @Resource
    private BankMapper bankMapper;
    @Resource
    private CwxxMapper  cwxxMapper;
    @Resource
    private CwxxYearMapper cwxxYearMapper;
    @Override
    public Long createDwxx(DwxxSaveReqVO createReqVO) {
        // 插入
        DwxxDO dwxx = BeanUtils.toBean(createReqVO, DwxxDO.class);
        dwxx.setId(YitIdHelper.nextId());
        /**
         * {
         * 	"id": 633249312886853,
         * 	"info": {
         * 		"nickname": "北京局管理员用户",
         * 		"deptId": "1171201049703"
         *        },
         * 	"userType": 2
         * }
         */
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        if (ObjectUtils.isNotEmpty(loginUser)) {
            String deptId = loginUser.getInfo().get("deptId");
            dwxx.setDwDeptId(deptId);
        } else {
            throw exception(DWXX_LOGIN_USER_EXISTS);
        }
        dwxxMapper.insert(dwxx);
        // 返回
        return dwxx.getId();
    }

    @Override
    public void updateDwxx(DwxxSaveReqVO updateReqVO) {
        // 校验存在
        validateDwxxExists(updateReqVO.getId());
        // 更新
        DwxxDO updateObj = BeanUtils.toBean(updateReqVO, DwxxDO.class);
        dwxxMapper.updateById(updateObj);
    }

    @Override
    public void deleteDwxx(Long id) {
        // 校验存在
        validateDwxxExists(id);
        // 删除
        dwxxMapper.deleteById(id);
    }

    private void validateDwxxExists(Long id) {
        if (dwxxMapper.selectById(id) == null) {
            throw exception(DWXX_NOT_EXISTS);
        }
    }

    @Override
    public DwxxDO getDwxx(Long id) {
        return dwxxMapper.selectById(id);
    }

    @Override
    public PageResult<DwxxDO> getDwxxPage(DwxxPageReqVO pageReqVO) {
        return dwxxMapper.selectPage(pageReqVO);
    }

    @Override
    public PageResult<DeliveryRespVO> getDeliveryPage(DeliveryPageReqVO pageReqVO) {
        if (StringUtils.isNotBlank(pageReqVO.getDwmc())) {
            pageReqVO.setDwmc(String.format("%s%s%s", "%", pageReqVO.getDwmc(), "%"));
        }
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        AdminUserDO adminUserDO = adminUserMapper.selectById(loginUserId);
        if (ObjectUtils.isEmpty(adminUserDO)) {
            throw exception(ERROR,"不存在用户信心信息");
        }
        Integer areaLevel = adminUserDO.getAreaLevel();
        Page<DeliveryRespVO> page =new  Page<DeliveryRespVO>();
        // 国家交易中心已选
        if (0==areaLevel) {
             if (pageReqVO.getAreaCode()!=null){
                 // 数据权限
                 pageReqVO.setAreaCode(AreaUtils.getAreaCode(pageReqVO.getAreaLevel(), pageReqVO.getAreaCode(), pageReqVO.getParentCode()));
             }
            page = dwxxMapper.selectWrapperPage(new Page<>(pageReqVO.getPageNo(), pageReqVO.getPageSize()), pageReqVO);

        }
        //省级交易中心与银行用户
        if (1==areaLevel) {
            String areaCode = adminUserDO.getAreaCode();
            pageReqVO.setAreaSheng(areaCode);
            if (ObjectUtils.isEmpty(pageReqVO.getAreaLevel())&&pageReqVO.getAreaCode()!=null) {
                if (XZQHLEVEL.SHI.getCode().equals(pageReqVO.getAreaLevel())) {
                    pageReqVO.setAreaShi(areaCode);
                }
                if (XZQHLEVEL.XIAN.getCode().equals(pageReqVO.getAreaLevel())) {
                    pageReqVO.setAreaXian(areaCode);
                }
            }
            //获取当前用户角色
            List<UserRoleDO> userRoleList = userRoleMapper.selectListByUserId(loginUserId);
            if (ObjectUtils.isEmpty(userRoleList)) {
                throw exception(ERROR,"不存在角色信息");
            }
            List<Long> roleIdList = userRoleList.stream().map(UserRoleDO::getRoleId).distinct().collect(Collectors.toList());
            if (roleIdList.contains(RoleIdEnum.BANK_ADMIN.getRoleId())) {
                page = dwxxMapper.selectWrapperPageForBank(new Page<>(pageReqVO.getPageNo(), pageReqVO.getPageSize()), pageReqVO);
            }else{
                page = dwxxMapper.selectWrapperPageForSheng(new Page<>(pageReqVO.getPageNo(), pageReqVO.getPageSize()), pageReqVO);
            }

        }
        return new PageResult<>(page.getRecords(), page.getTotal());
    }

    @Override
    public List<StoreRespVO> getStorePage(StorePageReqVO pageReqVO) {
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        if (ObjectUtils.isEmpty(loginUser)) {
            throw exception(ERROR, "不存在用户信息");
        }
        String areaLevel = loginUser.getInfo().get("areaLevel");
        Long loginUserId = loginUser.getId();
        //获取当前用户角色
        List<UserRoleDO> userRoleList = userRoleMapper.selectListByUserId(loginUserId);
        if (ObjectUtils.isEmpty(userRoleList)) {
            throw exception(ERROR, "不存在角色信息");
        }
        // 国家交易中心已选
        if ("0".equals(areaLevel)) {
            if (YXZT.NOT_SELECTED.getCode().equals(pageReqVO.getYxcf())) {
                List<StoreRespVO> storeListForNation = dwxxMapper.getStoreListForNation(pageReqVO);
                List<CaxxDO> caxxDOS = caxxMapper.selectListByKqId(pageReqVO.getKqId());
                List<StoreRespVO> selectedStoreS =new ArrayList<>();
                if (!ObjectUtils.isEmpty(caxxDOS)) {
                    selectedStoreS = BeanUtils.toBean(caxxDOS, StoreRespVO.class);
                }
                if (!ObjectUtils.isEmpty(storeListForNation)) {
                    if (!ObjectUtils.isEmpty(selectedStoreS)) {
                        storeListForNation.removeAll(selectedStoreS);
                    }
                    return  storeListForNation;
                }
            } else if (YXZT.SELECTED.getCode().equals(pageReqVO.getYxcf())||YXZT.OPTION.getCode().equals(pageReqVO.getYxcf())) {
                List<CaxxDO> caxxDOS = caxxMapper.selectListByKqId(pageReqVO.getKqId());
                if (!ObjectUtils.isEmpty(caxxDOS)) {
                    List<StoreRespVO> selectedStoreSForNation = BeanUtils.toBean(caxxDOS, StoreRespVO.class);
                    return  selectedStoreSForNation;
                }
            }
        }
        if ("1".equals(areaLevel)) {
            List<Long> roleIdList = userRoleList.stream().map(UserRoleDO::getRoleId).distinct().collect(Collectors.toList());
            if (roleIdList.contains(RoleIdEnum.BANK_ADMIN.getRoleId())) {// 银行已选
                if (YXZT.NOT_SELECTED.getCode().equals(pageReqVO.getYxcf())) {
                    List<StoreRespVO> storeListForBank= provinceMapper.getStoreListForProvince(pageReqVO.getKqId(),1);
                    List<StoreRespVO> storeListSelelct = bankMapper.getStoreListForBank(pageReqVO.getKqId());
                    if (!ObjectUtils.isEmpty(storeListForBank)) {
                        if (!ObjectUtils.isEmpty(storeListSelelct)){
                            storeListForBank.removeAll(storeListSelelct);
                        }
                        return  storeListForBank;
                    }
                } else if (YXZT.SELECTED.getCode().equals(pageReqVO.getYxcf())||YXZT.OPTION.getCode().equals(pageReqVO.getYxcf())) {
                    List<StoreRespVO> storeListForBank = bankMapper.getStoreListForBank(pageReqVO.getKqId());
                    return  storeListForBank;
                }
            } else {// 省已选
                if (YXZT.NOT_SELECTED.getCode().equals(pageReqVO.getYxcf())) {
                    List<StoreRespVO> storeListForProvice =new ArrayList<>();
                    List<CaxxDO> caxxDOS = caxxMapper.selectListByKqId(pageReqVO.getKqId());
                    List<StoreRespVO> storeListSelect= provinceMapper.getStoreListForProvince(pageReqVO.getKqId(),0);
                    if (!ObjectUtils.isEmpty(caxxDOS)) {
                        storeListForProvice = BeanUtils.toBean(caxxDOS, StoreRespVO.class);
                        if (!ObjectUtils.isEmpty(storeListSelect)) {
                            storeListForProvice.removeAll(storeListSelect);
                        }
                        return  storeListForProvice;
                    }

                } else if (YXZT.SELECTED.getCode().equals(pageReqVO.getYxcf())||YXZT.OPTION.getCode().equals(pageReqVO.getYxcf())) {
                    List<StoreRespVO> storeListForProvince = provinceMapper.getStoreListForProvince(pageReqVO.getKqId(),0);
                    return  storeListForProvince;
                }

            }
        }
        return null;
    }

    @Override
    @Transactional
    public void updateSelectedCaxx(CaxxUpdateReqVO updateReqVO) {
        Long loginUserId = SecurityFrameworkUtils.getLoginUser().getId();
        //获取当前用户角色
        List<UserRoleDO> userRoleList = userRoleMapper.selectListByUserId(loginUserId);
        if (ObjectUtils.isEmpty(userRoleList)) {
            throw exception(ERROR,"不存在角色信息");
        }
        String areaLevel = updateReqVO.getAreaLevel();
        // 国家交易中心已选
        if ("0".equals(areaLevel)) {
            createCaAndKuAndDWForSheng(updateReqVO);
        }
        if ("1".equals(areaLevel)) {
            List<Long> roleIdList = userRoleList.stream().map(UserRoleDO::getRoleId).distinct().collect(Collectors.toList());
            if (roleIdList.contains(RoleIdEnum.BANK_ADMIN.getRoleId())) {// 银行已选
                List<BankDO> bankDOS=new ArrayList<>();
                for (Long caId:updateReqVO.getIds()){
                    BankDO bankDO = new BankDO();
                    bankDO.setCaId(caId);
                    bankDO.setKqId(updateReqVO.getKqId());
                    bankDO.setId(YitIdHelper.nextId());
                }
                if (!ObjectUtils.isEmpty(bankDOS)){
                    bankMapper.insertBatch(bankDOS);
                }
            }else{// 省已选
                List<ProvinceDO> provinceDOS=new ArrayList<>();
                for (Long caId:updateReqVO.getIds()){
                    ProvinceDO province = new ProvinceDO();
                    province.setCaId(caId);
                    province.setKqId(updateReqVO.getKqId());
                    province.setId(YitIdHelper.nextId());
                }
                if (!ObjectUtils.isEmpty(provinceDOS)){
                    provinceMapper.insertBatch(provinceDOS);
                }
            }
        }
    }

    @Override
    @Transactional
    public void revertCaxx(CaxxUpdateReqVO updateReqVO) {
        Long loginUserId = SecurityFrameworkUtils.getLoginUser().getId();
        //获取当前用户角色
        List<UserRoleDO> userRoleList = userRoleMapper.selectListByUserId(loginUserId);
        if (ObjectUtils.isEmpty(userRoleList)) {
            throw exception(ERROR,"不存在角色信息");
        }
        String areaLevel = updateReqVO.getAreaLevel();
        // 国家交易中心已选
        if ("0".equals(areaLevel)) {
            caxxMapper.deleteBatchIds(updateReqVO.getIds());
            List<CaxxDO> caxxDOS = caxxMapper.selectListByKqId(updateReqVO.getKqId());
            if (ObjectUtils.isEmpty(caxxDOS)){
                dwxxMapper.deleteById(updateReqVO.getDeptId());
                kqxxMapper.deleteById(updateReqVO.getKqId());
                CwxxDO cwxxDO = cwxxMapper.selectBYKqId(updateReqVO.getKqId());
                if (!ObjectUtil.isEmpty(cwxxDO)){
                    Long cwxxDOId = cwxxDO.getId();
                    cwxxMapper.deleteById(cwxxDOId);
                    cwxxYearMapper.deleteListByCwxxId(cwxxDOId);
                }
            }
        }
        if ("1".equals(areaLevel)) {
            List<Long> roleIdList = userRoleList.stream().map(UserRoleDO::getRoleId).distinct().collect(Collectors.toList());
            if (roleIdList.contains(RoleIdEnum.BANK_ADMIN.getRoleId())) {
                // 银行已选
                bankMapper.deleteListByKqIdAndCaIds(updateReqVO.getKqId(), updateReqVO.getIds());
            }else{
                // 省已选
                provinceMapper.deleteListByKqIdAndCaIds(updateReqVO.getKqId(), updateReqVO.getIds());
                KqxxDO kqxxDO = kqxxMapper.selectById(updateReqVO.getKqId());
                if (ObjectUtils.isEmpty(kqxxDO)){
                    throw exception(ERROR,"库区信息不存在");
                }
                Integer sjly = kqxxDO.getSjly();
                if (sjly!=null&&sjly==0){
                    //新建的库区，如果省退回全部的仓房则删除库区的全部信息
                    List<ProvinceDO> provinceDOS = provinceMapper.selectListByKqId(updateReqVO.getKqId());
                    if (ObjectUtils.isEmpty(provinceDOS)){
                        dwxxMapper.deleteById(updateReqVO.getDeptId());
                        kqxxMapper.deleteById(updateReqVO.getKqId());
                        caxxMapper.deleteListByKqId(updateReqVO.getKqId());
                        CwxxDO cwxxDO = cwxxMapper.selectBYKqId(updateReqVO.getKqId());
                        if (!ObjectUtil.isEmpty(cwxxDO)){
                            Long cwxxDOId = cwxxDO.getId();
                            cwxxMapper.deleteById(cwxxDOId);
                            cwxxYearMapper.deleteListByCwxxId(cwxxDOId);
                        }
                    }
                }
            }
        }

    }
    @Transactional
    public void createCaAndKuAndDWForSheng(CaxxUpdateReqVO updateReqVO) {
        if (updateReqVO.getKqId() != null) {
            KqxxDO kqxxDOExist = kqxxMapper.selectById(updateReqVO.getKqId());
            if (ObjectUtils.isEmpty(kqxxDOExist)) {
                CodingKqxxDO codingKqxxDO = codingKqxxMapper.selectById(updateReqVO.getKqId());
                if (ObjectUtils.isEmpty(codingKqxxDO)) {
                    throw exception(ERROR, "当前库区信息不存在！");
                }
                KqxxDO kqxxDO = BeanUtils.toBean(codingKqxxDO, KqxxDO.class);
                String shengAll = kqxxDO.getSheng();
                String[] parts = shengAll.split(",");
                for (int i = 0; i < parts.length; i++) {
                    parts[i] = parts[i].trim(); // 去除空格
                }
                String province = null;
                String city = null;
                String county = null;
                if (parts.length != 0) {
                    province = parts[0];
                    if (parts.length == 3) {
                        city = parts[1];
                        county = parts[2];
                    }
                    if (parts.length == 2) {
                        city = parts[1];
                    }
                }
                List<AreaLevelDO> areaLevelDOS = areaLevelMapper.selectListByName(Arrays.asList(province, city, county));
                if (ObjectUtils.isEmpty(areaLevelDOS)) {
                    throw exception(ERROR, "当前库区所在的行政区划不存在！");
                }
                Map<String, String> map = areaLevelDOS.stream().collect(Collectors.toMap(AreaLevelDO::getAreaName, AreaLevelDO::getAreaCode, (x, y) -> x));
                if (province != null) {
                    String shengCode = map.get(province);
                    kqxxDO.setSheng(shengCode);
                }
                if (city != null) {
                    String shiCode = map.get(city);
                    kqxxDO.setShi(shiCode);
                }
                if (county != null) {
                    String xianCode = map.get(county);
                    kqxxDO.setXian(xianCode);
                }
                kqxxDO.setSjly(1);
                kqxxMapper.insert(kqxxDO);
            }
        }
        if (updateReqVO.getDeptId() != null) {
            DwxxDO dwxxDOExist = dwxxMapper.selectById(updateReqVO.getDeptId());
            if (ObjectUtils.isEmpty(dwxxDOExist)) {
                CodingDwxxDO codingDwxxDO = codingDwxxMapper.selectById(updateReqVO.getDeptId());
                if (ObjectUtils.isEmpty(codingDwxxDO)) {
                    throw exception(ERROR, "当前单位信息不存在！");
                }
                DwxxDO dwxxDO = BeanUtils.toBean(codingDwxxDO, DwxxDO.class);
                dwxxDO.setSjly(1);
                dwxxMapper.insert(dwxxDO);
            }
        }
        if (!ObjectUtils.isEmpty(updateReqVO.getIds())) {
            List<CodingCaxxDO> codingCaxxDOS = codingCaxxMapper.selectList(CodingCaxxDO::getId, updateReqVO.getIds());
            if (ObjectUtils.isEmpty(codingCaxxDOS)) {
                throw exception(ERROR, "所选仓房信息不存在！");
            }
            List<CaxxDO> caxxDOSExist = caxxMapper.selectListByKqId(updateReqVO.getKqId());
            List<CaxxDO> caxxDOS = BeanUtils.toBean(codingCaxxDOS, CaxxDO.class);
            if (!ObjectUtils.isEmpty(caxxDOS)) {
                if (!ObjectUtils.isEmpty(caxxDOSExist)) {
                    List<Long> idsExist = caxxDOSExist.stream().map(CaxxDO::getId).collect(Collectors.toList());
                     caxxDOS.removeIf(caxxDO -> idsExist.contains(caxxDO.getId()));
                }
                if (!ObjectUtils.isEmpty(caxxDOS)){
                    caxxDOS.stream().forEach(s -> {
                        s.setSjly(1);
                        s.setYxcf(1);
                    });
                    caxxMapper.insertBatch(caxxDOS);
                }
            }
        }
    }



}