package com.unis.module.coding.service.task;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.yitter.idgen.YitIdHelper;
import com.unis.framework.common.pojo.PageParam;
import com.unis.framework.common.util.object.BeanUtils;
import com.unis.framework.security.core.LoginUser;
import com.unis.framework.security.core.util.SecurityFrameworkUtils;
import com.unis.module.coding.controller.admin.task.vo.*;
import com.unis.module.coding.dal.dataobject.bank.BankDO;
import com.unis.module.coding.dal.dataobject.caxx.CaxxDO;
import com.unis.module.coding.dal.dataobject.cwxx.CwxxDO;
import com.unis.module.coding.dal.dataobject.cwxxyear.CwxxYearDO;
import com.unis.module.coding.dal.dataobject.dk.DkDO;
import com.unis.module.coding.dal.dataobject.dwxx.DwxxDO;
import com.unis.module.coding.dal.dataobject.kqxx.KqxxDO;
import com.unis.module.coding.dal.dataobject.province.ProvinceDO;
import com.unis.module.coding.dal.mysql.bank.BankMapper;
import com.unis.module.coding.dal.mysql.caxx.CaxxMapper;
import com.unis.module.coding.dal.mysql.cwxx.CwxxMapper;
import com.unis.module.coding.dal.mysql.cwxxyear.CwxxYearMapper;
import com.unis.module.coding.dal.mysql.dk.DkMapper;
import com.unis.module.coding.dal.mysql.dwxx.DwxxMapper;
import com.unis.module.coding.dal.mysql.kqxx.KqxxMapper;
import com.unis.module.coding.dal.mysql.province.ProvinceMapper;
import com.unis.module.core.enums.ErrorCodeConstants;
import com.unis.module.core.enums.project.DeliveryHyState;
import com.unis.module.system.api.permission.PermissionApi;
import com.unis.module.system.enums.permission.RoleCodeEnum;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import static com.unis.framework.common.exception.util.ServiceExceptionUtil.exception;


/**
 * 省已选 Service 实现类
 *
 * @author 超级管理员
 */
@Service
@Validated
@Slf4j
public class TaskServiceImpl implements TaskService {

    @Resource
    private DkMapper dkMapper;
    @Resource
    private PermissionApi permissionApi;
    @Resource
    private ProvinceMapper provinceMapper;
    @Resource
    private CaxxMapper caxxMapper;
    @Resource
    private KqxxMapper kqxxMapper;
    @Resource
    private DwxxMapper dwxxMapper;
    @Resource
    private CwxxMapper cwxxMapper;
    @Resource
    private CwxxYearMapper cwxxYearMapper;
    @Resource
    private BankMapper bankMapper;

    @Override
    public IPage<TaskRespVO> getTaskList(TaskPageReqVO pageReqVO) {
        // 根据角色不同，查询不同数据
        addParamDeptIds(pageReqVO);

        IPage<TaskRespVO> page = new Page<>(pageReqVO.getPageNo(), pageReqVO.getPageSize());
        dkMapper.getTaskList(page, pageReqVO);

        return page;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean getTasks(TaskPageReqVO reqVO) {

        //校验是登录人可选的任务
        reqVO.setListType(TaskPageReqVO.LIST_TYPE_TASK1);
        verifyTask(reqVO);

        Long loginUserId = SecurityFrameworkUtils.getLoginUserIdNotNull();
        // 查询登录人角色标识
        Set<String> roleCodes = permissionApi.getUserRoleCodeListByUserIdFromCache(loginUserId);

        // 更新核验状态为核验中,核验人员 id,核验人员名
        // 省级核验人员
        if(roleCodes.contains(RoleCodeEnum.PROVINCIAL_VERIFICATION_PERSONNEL.getCode())) {
            reqVO.getKqIds().forEach(kqId -> {
                ProvinceDO provinceDO = new ProvinceDO()
                        .setHyState(reqVO.getHyState() == null ? DeliveryHyState.WAIT_HY.getCode() : reqVO.getHyState())
                        .setHyryId(SecurityFrameworkUtils.getLoginUserId())
                        .setHyryName(SecurityFrameworkUtils.getLoginUserNickname());
                provinceMapper.updateByKqId(provinceDO, kqId);
            });
        }
        // 银行核验人员
        if(roleCodes.contains(RoleCodeEnum.VERIFICATION_PERSONNEL.getCode())) {
            reqVO.getKqIds().forEach(kqId -> {
                BankDO bankDO = new BankDO()
                        .setHyState(reqVO.getHyState() == null ? DeliveryHyState.WAIT_HY.getCode() : reqVO.getHyState())
                        .setHyryId(SecurityFrameworkUtils.getLoginUserId())
                        .setHyryName(SecurityFrameworkUtils.getLoginUserNickname());
                bankMapper.updateByKqId(bankDO, kqId);
            });
        }
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean hyKqxx(TaskPageReqVO reqVO) {
        //校验是登录人可选的任务
        reqVO.setListType(TaskPageReqVO.LIST_TYPE_TASK2);
        verifyTask(reqVO);

        Long loginUserId = SecurityFrameworkUtils.getLoginUserIdNotNull();
        // 查询登录人角色标识
        Set<String> roleCodes = permissionApi.getUserRoleCodeListByUserIdFromCache(loginUserId);

        // 更新核验状态为核验中,核验人员 id,核验人员名
        // 省级核验人员
        if(roleCodes.contains(RoleCodeEnum.PROVINCIAL_VERIFICATION_PERSONNEL.getCode())) {
            reqVO.getKqIds().forEach(kqId -> {
                ProvinceDO provinceDO = new ProvinceDO()
                        .setHyState(reqVO.getHyState())
                        .setHyryId(SecurityFrameworkUtils.getLoginUserId())
                        .setHyryName(SecurityFrameworkUtils.getLoginUserNickname());
                provinceMapper.updateByKqId(provinceDO,kqId);
            });
        }
        // 银行核验人员
        if(roleCodes.contains(RoleCodeEnum.VERIFICATION_PERSONNEL.getCode())) {
            reqVO.getKqIds().forEach(kqId -> {
                BankDO bankDO = new BankDO()
                        .setHyState(reqVO.getHyState())
                        .setHyryId(SecurityFrameworkUtils.getLoginUserId())
                        .setHyryName(SecurityFrameworkUtils.getLoginUserNickname());
                bankMapper.updateByKqId(bankDO, kqId);
            });
        }
        return true;
    }

    @Override
    public List<TaskCaxxRespVO> getCaxxByKqId(TaskPageReqVO reqVO) {

        List<CaxxDO> caxxDOS = caxxMapper.selectListByKqId(reqVO.getKqId());

        if(ObjectUtils.isEmpty(caxxDOS)){
           return BeanUtils.toBean(caxxDOS, TaskCaxxRespVO.class);
        }

        return new ArrayList<>();
    }

    @Override
    public TaskKqxxRespVO getKqxxByKqId(TaskPageReqVO reqVO) {

        KqxxDO kqxxDO = kqxxMapper.selectById(reqVO.getKqId());

        DwxxDO dwxxDO = dwxxMapper.selectById(kqxxDO.getDwid());

        if(ObjectUtils.isNotEmpty(kqxxDO)) {
            TaskKqxxRespVO kqxx = BeanUtils.toBean(kqxxDO, TaskKqxxRespVO.class);
            kqxx.setDwmc(dwxxDO.getDwmc());
            kqxx.setTyxydm(dwxxDO.getTyxydm());
            kqxx.setQyxz(dwxxDO.getQyxz());
            kqxx.setJd(dwxxDO.getJd());
            kqxx.setWd(dwxxDO.getWd());
            return kqxx;
        }

        return null;
    }

    @Override
    public TaskCwxxRespVO getCwxxByKqId(TaskPageReqVO reqVO) {

        CwxxDO cwxxDO = cwxxMapper.selectBYKqId(reqVO.getKqId());

        if(ObjectUtils.isEmpty(cwxxDO)){
            return null;
        }

        List<CwxxYearDO> cnxxList = cwxxYearMapper.selectByCwxxId(cwxxDO.getId());

        TaskCwxxRespVO cwxxRespVO = BeanUtils.toBean(cwxxDO, TaskCwxxRespVO.class);

        cwxxRespVO.setCnxxList(BeanUtils.toBean(cnxxList, TaskCnxxRespVO.class));

        return cwxxRespVO;
    }

    @Override
    public TaskDkxxRespVO getDkxxByKqId(TaskPageReqVO reqVO) {
        DkDO dkDO = dkMapper.selectByKqIdAndUserId(reqVO.getKqId(), SecurityFrameworkUtils.getLoginUserId());

        if(ObjectUtils.isEmpty(dkDO)){
            return null;
        }
        return BeanUtils.toBean(dkDO, TaskDkxxRespVO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean setDkxxByKqId(TaskDkxxSaveVO saveVO) {

        if(ObjectUtils.isEmpty(saveVO) || ObjectUtils.isEmpty(saveVO.getKqId())){
            throw exception(ErrorCodeConstants.APP_PARAM_ERROR);
        }
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        DkDO dkDO = dkMapper.selectByKqIdAndUserId(saveVO.getKqId(), loginUserId );

        if(ObjectUtils.isEmpty(saveVO.getId())){
            if(ObjectUtils.isNotEmpty(dkDO)){
                throw exception(ErrorCodeConstants.APP_TASK_RECEIVE_ERROR_RETRY);
            }

            // 查询登录人角色标识,填充[打卡类型]
            setDkType(saveVO, loginUserId);

            //新增
            DkDO saveDo = getDkDO(saveVO, loginUserId);
            dkMapper.insert(saveDo);
        }else{
            //变更
            if(ObjectUtils.isNotEmpty(dkDO) && dkDO.getId().equals(saveVO.getId())){
                dkMapper.updateById(BeanUtils.toBean(saveVO, DkDO.class));
            }else{
                throw exception(ErrorCodeConstants.APP_PARAM_ERROR);
            }
        }

        return null;
    }

    @NotNull
    private static DkDO getDkDO(TaskDkxxSaveVO saveVO, Long loginUserId) {
        DkDO saveDo = BeanUtils.toBean(saveVO, DkDO.class);
        saveDo.setId(YitIdHelper.nextId());// 雪花id
        saveDo.setUserId(loginUserId);
        saveDo.setDeptId(SecurityFrameworkUtils.getLoginUserDeptId());
        return saveDo;
    }

    private void setDkType(TaskDkxxSaveVO saveVO, Long loginUserId) {
        Set<String> roleCodes = permissionApi.getUserRoleCodeListByUserIdFromCache(loginUserId);
        if(roleCodes.contains(RoleCodeEnum.PROVINCIAL_VERIFICATION_PERSONNEL.getCode())) {
            saveVO.setDkType(TaskDkxxSaveVO.PROVINCIAL_VERIFICATION_PERSONNEL_DK_TYPE);
        }else if(roleCodes.contains(RoleCodeEnum.VERIFICATION_PERSONNEL.getCode())) {
            saveVO.setDkType(TaskDkxxSaveVO.VERIFICATION_PERSONNEL_DK_TYPE);
        }else{
            throw exception(ErrorCodeConstants.APP_ACCOUNT_PERMISSION_DENIED);
        }
    }

    private void verifyTask(TaskPageReqVO reqVO) {

        TaskPageReqVO pageReqVO = new TaskPageReqVO();
        pageReqVO.setKqIds(reqVO.getKqIds());
        pageReqVO.setListType(reqVO.getListType());
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);

        IPage<TaskRespVO> taskList = getTaskList(pageReqVO);

        if(taskList.getTotal() != reqVO.getKqIds().size()) {
            throw exception(ErrorCodeConstants.APP_TASK_RECEIVE_ERROR);
        }
    }

    private void addParamDeptIds(@NotNull TaskPageReqVO pageReqVO) {

        Long loginUserId = SecurityFrameworkUtils.getLoginUserIdNotNull();
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        // 查询登录人角色标识
        Set<String> roleCodes = permissionApi.getUserRoleCodeListByUserIdFromCache(loginUserId);

        // 查询待核验列表
        if(pageReqVO.getListType() == null){
            throw exception(ErrorCodeConstants.APP_PARAM_ERROR);
        }else if(1 == pageReqVO.getListType()) {
            pageReqVO.setHyState(DeliveryHyState.WAIT.getCode());
        } else if(2 == pageReqVO.getListType()) {
            pageReqVO.setHyryId(loginUserId);
        } else {
            throw exception(ErrorCodeConstants.APP_PARAM_ERROR);
        }

        // 省级核验人员
        if(roleCodes.contains(RoleCodeEnum.PROVINCIAL_VERIFICATION_PERSONNEL.getCode())) {
            if(loginUser != null) {
                String areaLevel = loginUser.getInfo().get("areaLevel");
                if("1".equals(areaLevel)) {
                    pageReqVO.setSheng(loginUser.getInfo().get("shengCode"));
                    pageReqVO.setTableName(TaskPageReqVO.TABLE_NAME_DELIVERY_PROVINCE);
                } else {
                    throw exception(ErrorCodeConstants.APP_ACCOUNT_PERMISSION_DENIED);
                }
            }
            return;
        }
        // 银行核验人员
        if(roleCodes.contains(RoleCodeEnum.VERIFICATION_PERSONNEL.getCode())) {
            if(loginUser != null) {
                String areaLevel = loginUser.getInfo().get("areaLevel");
                if("1".equals(areaLevel)) {
                    pageReqVO.setSheng(loginUser.getInfo().get("sheng_code"));
                    //银行需要查询自己银行选定的
                    pageReqVO.setDeptId(loginUser.getInfo().get("deptId"));
                    pageReqVO.setTableName(TaskPageReqVO.TABLE_NAME_DELIVERY_BANK);
                } else {
                    throw exception(ErrorCodeConstants.APP_ACCOUNT_PERMISSION_DENIED);
                }
            }
            return;
        }

        throw exception(ErrorCodeConstants.APP_ACCOUNT_PERMISSION_DENIED);
    }
}